from urllib.parse import urljoin

import pynyaa
import requests
from bs4 import BeautifulSoup

ANIMETOSHO_FEED_URL = "https://animetosho.org/feed/json"


def get_nyaa_url(url):
    """Get Nyaa torrent link from URL

    Args:
        url (str): URL to get Nyaa torrent link
    """

    nyaa = pynyaa.get(url)
    parsed_url = str(nyaa.torrent_file)

    return parsed_url


def get_animetosho_url(url):
    """Get AnimeTosho torrent link from URL

    Args:
        url (str): URL to get AnimeTosho torrent link
    """

    # Start by getting the webpage, so we can get a title
    r = requests.get(url)
    soup = BeautifulSoup(r.content, "html.parser")
    titles = soup.find_all("h2", attrs={"id": "title"})

    if len(titles) == 0:
        raise Exception("Could not find torrent name in AnimeTosho webpage")

    if len(titles) > 1:
        raise Exception("More than one torrent title in AnimeTosho webpage")

    title = titles[0].text

    # Fantastic, we have a title. Now query API
    query_url = urljoin(ANIMETOSHO_FEED_URL, f"?t=search&q={title}")
    r = requests.get(query_url)
    j = r.json()

    # Loop over, make sure the link matches the URL and get a torrent link out
    parsed_url = None
    for i in j:

        if parsed_url is not None:
            continue

        link = i.get("link", None)
        if link == url:
            parsed_url = i.get("torrent_url", None)

    return parsed_url



import logging
import math
import os
import shutil
import sys
from logging.handlers import RotatingFileHandler

import colorlog


def setup_logger(
    log_level,
    log_dir="logs",
    log_name="SeaDexArr",
    max_logs=9,
):
    """
    Set up the logger.

    Parameters:
        log_level (str): The log level to use
        log_dir (str): Directory for log files.
            Defaults to "logs"
        log_name (str): The name of the log file.
            Defaults to "SeaDexArr"
        max_logs (int): Maximum number of log files to keep.
            Defaults to 9

    Returns:
        A logger object for logging messages.
    """

    if os.environ.get("DOCKER_ENV"):
        log_dir = os.path.join("/config", log_dir)
    else:
        log_dir = os.path.join(os.getcwd(), log_dir)

    # Create the log directory if it doesn't exist
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    # Define the log file path
    log_file = os.path.join(log_dir, f"{log_name}.log")

    # Check if a log file already exists. Copy, then remove to avoid I/O errors
    if os.path.isfile(log_file):
        for i in range(max_logs - 1, 0, -1):
            old_log = os.path.join(f"{log_dir}", f"{log_name}.log.{i}")
            new_log = os.path.join(f"{log_dir}", f"{log_name}.log.{i + 1}")
            if os.path.exists(old_log):
                if os.path.exists(new_log):
                    os.remove(new_log)
                shutil.copy(old_log, new_log)
                os.remove(old_log)

        shutil.copy(log_file, os.path.join(log_dir, f"{log_name}.log.1"))
        os.remove(log_file)

    # Create a logger object with the script name
    logger = logging.getLogger(log_name)
    logger.propagate = False

    # Set the log level based on the provided parameter
    log_level = log_level.upper()
    if log_level == "DEBUG":
        logger.setLevel(logging.DEBUG)
    elif log_level == "INFO":
        logger.setLevel(logging.INFO)
    elif log_level == "WARNING":
        logger.setLevel(logging.WARNING)
    elif log_level == "CRITICAL":
        logger.setLevel(logging.CRITICAL)
    else:
        logger.critical(f"Invalid log level '{log_level}', defaulting to 'INFO'")
        logger.setLevel(logging.INFO)

    # Define the log message format for the log files
    logfile_formatter = logging.Formatter(
        fmt="%(asctime)s %(levelname)s: %(message)s", datefmt="%m/%d/%y %I:%M %p"
    )

    # Create a RotatingFileHandler for log files
    handler = RotatingFileHandler(
        log_file, delay=True, mode="w", encoding="utf-8", backupCount=max_logs
    )
    handler.setFormatter(logfile_formatter)

    # Add the file handler to the logger
    logger.addHandler(handler)

    # Configure console logging with the specified log level
    console_handler = colorlog.StreamHandler(sys.stdout)
    if log_level == "DEBUG":
        console_handler.setLevel(logging.DEBUG)
    elif log_level == "INFO":
        console_handler.setLevel(logging.INFO)
    elif log_level == "CRITICAL":
        console_handler.setLevel(logging.CRITICAL)

    # Add the console handler to the logger
    console_handler.setFormatter(
        colorlog.ColoredFormatter("%(log_color)s%(levelname)s: %(message)s")
    )
    logger.addHandler(console_handler)

    # Overwrite previous logger if exists
    logging.getLogger(log_name).handlers.clear()
    logging.getLogger(log_name).addHandler(handler)
    logging.getLogger(log_name).addHandler(console_handler)

    return logger


def centred_string(
    str_to_centre,
    total_length=80,
    str_prefix="",
):
    """Centre string for a logger

    Args:
        str_to_centre: String to centre
        total_length: Total length of the string. Defaults to 80.
        str_prefix: Will include this at the start of any string. Defaults to ""
    """

    remaining_length = total_length - len(str_to_centre)
    left_side_length = math.floor(remaining_length / 2)
    right_side_length = remaining_length - left_side_length

    return f"{str_prefix}|{' ' * left_side_length} {str_to_centre} {' ' * right_side_length}|"


def left_aligned_string(
    str_to_align,
    total_length=80,
    str_prefix="",
):
    """Left-align string for a logger

    Args:
        str_to_align: String to align
        total_length: Total length of the string
        str_prefix: Will include this at the start of any string. Defaults to ""
    """

    remaining_length = total_length - len(str_to_align)
    left_side_length = 1
    right_side_length = remaining_length - left_side_length

    return f"{str_prefix}|{' ' * left_side_length} {str_to_align} {' ' * right_side_length}|"



import argparse
import os
import time
import traceback
from datetime import datetime, timedelta

from seadexarr import SeaDexSonarr, SeaDexRadarr, setup_logger

ALLOWED_ARRS = [
    "radarr",
    "sonarr",
]

# Define the parser and parse args
parser = argparse.ArgumentParser(description="SeaDexArr")
parser.add_argument("--arr", action="store", dest="arr")
args = parser.parse_args()

# See if we're doing a one-run, or scheduling
arr = args.arr

running_schedule = False
if arr is None:
    running_schedule = True

# Set up config file location
config_dir = os.getenv("CONFIG_DIR", os.getcwd())
config = os.path.join(config_dir, "config.yml")

if running_schedule:

    # Get how often to run things
    schedule_time = float(os.getenv("SCHEDULE_TIME", 6))

    while True:

        logger = setup_logger(log_level="INFO")
        logger.info(f"Running in scheduled mode")

        present_time = datetime.now().strftime("%H:%M")
        logger.info(f"Time is {present_time}. Starting run")

        # Run both Radarr and Sonarr syncs, catching
        # errors if they do arise. Split them up
        # so one crashing doesn't ruin the other
        try:
            sdr = SeaDexRadarr(
                config=config,
                logger=logger,
            )
            sdr.run()
        except Exception:
            tb = traceback.format_exc()
            for line in tb.splitlines():
                logger.warning(line)

        try:
            sds = SeaDexSonarr(
                config=config,
                logger=logger,
            )
            sds.run()
        except Exception:
            tb = traceback.format_exc()
            for line in tb.splitlines():
                logger.warning(line)

        next_run_time = datetime.now() + timedelta(hours=schedule_time)
        next_run_time = next_run_time.strftime("%H:%M")
        logger.info(f"Run complete! Will run again at {next_run_time}")

        # Good job, have a rest
        time.sleep(schedule_time * 3600)

# Else we're in a single run mode
else:

    logger = setup_logger(log_level="INFO")

    if arr in ALLOWED_ARRS:

        try:
            if arr == "radarr":
                sdr = SeaDexRadarr(
                    config=config,
                    logger=logger,
                )
                sdr.run()

            elif arr == "sonarr":
                sds = SeaDexSonarr(
                    config=config,
                    logger=logger,
                )
                sds.run()
        except Exception:
            tb = traceback.format_exc()
            for line in tb.splitlines():
                logger.warning(line)

    else:
        logger.warning(f"Arr {arr} unknown")
